<?php

namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\aws_cloud\Service\AwsEc2ServiceInterface;
use Drupal\cloud\Form\CloudContentForm;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\cloud\Service\EntityLinkRendererInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The base form class for the AWS Cloud content form.
 */
class AwsCloudContentForm extends CloudContentForm {

  /**
   * The AWS EC2 SErvice.
   *
   * @var \Drupal\aws_cloud\Service\AwsEc2ServiceInterface
   */
  protected $awsEc2Service;

  /**
   * @var \Drupal\cloud\Service\EntityLinkRendererInterface
   */
  protected $entityLinkRenderer;

  /**
   * The Entity Type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * AwsCloudContentForm constructor.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $manager
   *   The Entity Manager.
   * @param \Drupal\aws_cloud\Service\AwsEc2ServiceInterface $aws_ec2_service
   *   The AWS EC2 Service.
   * @param \Drupal\Core\Messenger\Messenger $messenger
   *   The Messenger Service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository service.
   * @param \Drupal\cloud\Service\EntityLinkRendererInterface $entity_link_renderer
   *   The entity link render service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The Entity Type Manager.
   */
  public function __construct(EntityManagerInterface $manager,
                              AwsEc2ServiceInterface $aws_ec2_service,
                              Messenger $messenger,
                              EntityRepositoryInterface $entity_repository,
                              EntityLinkRendererInterface $entity_link_renderer,
                              EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($manager, $entity_repository, $messenger);
    $this->awsEc2Service = $aws_ec2_service;
    $this->entityLinkRenderer = $entity_link_renderer;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.manager'),
      $container->get('aws_cloud.ec2'),
      $container->get('messenger'),
      $container->get('entity.repository'),
      $container->get('entity.link_renderer'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // Copy values of the form elements whose type are item to entity.
    // If not, the properties corresponding to the form elements
    // will be saved as NULL.
    $this->copyFormItemValues($form);

    parent::save($form, $form_state);
  }

  /**
   *
   */
  protected function copyFormItemValues(array $form) {
    $original_entity = $this->manager
      ->getStorage($this->entity->getEntityTypeId())
      ->load($this->entity->id());

    $item_field_names = [];
    foreach ($form as $name => $item) {
      if (!is_array($item)) {
        continue;
      }

      if (isset($item['#type']) && $item['#type'] == 'item') {
        $item_field_names[] = $name;
      }

      if (isset($item['#type']) && $item['#type'] == 'details') {
        foreach ($item as $sub_item_name => $sub_item) {
          if (is_array($sub_item) && isset($sub_item['#type']) && $sub_item['#type'] == 'item') {
            $item_field_names[] = $sub_item_name;
          }
        }
      }
    }

    foreach ($item_field_names as $item_field_name) {
      $item_field_value = $original_entity->get($item_field_name)->value;
      if ($item_field_value !== NULL) {
        $this->entity->set($item_field_name, $item_field_value);
      }
    }
  }

}
