<?php

// Created by yas 2016/05/30.
namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\Language;

/**
 * Form controller for the CloudScripting entity edit forms.
 *
 * @ingroup aws_cloud
 */
class ImageEditForm extends AwsCloudContentForm {

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    /* @var $entity \Drupal\aws_cloud\Entity\Ec2\Image */
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;

    $weight = -50;

    $form['image'] = [
      '#type' => 'details',
      '#title' => $this->t('Image'),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['image']['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->getName(),
      '#required'      => TRUE,
    ];

    $form['image']['description'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Description'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->getDescription(),
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['image']['ami_name'] = [
      '#type'          => 'item',
      '#title'         => $this->t('AMI Name: '),
      '#markup'        => $entity->getAmiName(),
    ];

    $form['image']['image_id'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Image ID: '),
      '#markup'        => $entity->getImageId(),
    ];

    $form['image']['owner'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Owner: '),
      '#markup'        => $entity->getImageOwnerId(),
    ];

    $form['image']['source'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Source: '),
      '#markup'        => $entity->getSource(),
    ];

    $form['image']['status'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Status: '),
      '#markup' => $entity->getStatus(),
    ];

    $form['image']['state_reason'] = [
      '#type'          => 'item',
      '#title'         => $this->t('State Reason: '),
      '#markup' => $entity->getStateReason(),
    ];

    $form['type'] = [
      '#type' => 'details',
      '#title' => $this->t('Type: '),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['type']['platform'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Platform: '),
      '#markup'        => $entity->getPlatform(),
    ];

    $form['type']['architecture'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Architecture: '),
      '#markup'        => $entity->getArchitecture(),
    ];

    $form['type']['virtualization_type'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Virtualization Type: '),
      '#markup'        => $entity->getVirtualizationType(),
    ];

    $form['type']['product_code'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Product Code: '),
      '#markup'        => $entity->getProductCode(),
    ];

    $form['type']['image_type'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Image Type: '),
      '#markup'        => $entity->getImageType(),
    ];

    $form['device'] = [
      '#type' => 'details',
      '#title' => $this->t('Device: '),
      '#open' => TRUE,
      '#weight' => $weight++,
    ];

    $form['device']['root_device_name'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Root Device Name: '),
      '#markup'        => $entity->getRootDeviceName(),
    ];

    $form['device']['root_device_type'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Root Device Type: '),
      '#markup'        => $entity->getRootDeviceType(),
    ];

    $form['device']['kernel_id'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Kernel ID: '),
      '#markup'        => $entity->getKernelId(),
    ];

    $form['device']['ramdisk_id'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Ramdisk ID: '),
      '#markup'        => $entity->getRamdiskId(),
    ];

    $form['device']['block_devices'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Block Devices: '),
      '#markup'        => $entity->getBlockDevices(),
    ];

    $form['others'] = [
      '#type'          => 'details',
      '#title'         => $this->t('Others'),
      '#open'          => FALSE,
      '#weight'        => $weight++,
    ];

    $form['others']['cloud_context'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Cloud ID: '),
      '#markup'        => !$entity->isNew()
      ? $entity->getCloudContext()
      : $cloud_context,
    ];

    $form['others']['visibility'] = [
      '#type'          => 'item',
      '#title'         => $this->t('Visibility: '),
      '#markup' => $entity->getVisibility(),
    ];

    $form['others']['langcode'] = [
      '#title'         => t('Language'),
      '#type'          => 'language_select',
      '#default_value' => $entity->getUntranslated()->language()->getId(),
      '#languages'     => Language::STATE_ALL,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => FALSE,
    ];

    $form['others']['user_id'] = $form['user_id'];
    unset($form['user_id']);

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);
    $form['actions']['#weight'] = $weight++;

    return $form;
  }

}
