<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

use Drupal\Tests\aws_cloud\Functional\Utils;
use Drupal\Component\Serialization\Yaml;

// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Updated by yas 2016/05/25
// Updated by yas 2016/05/24
// Created by yas 2016/05/23.
define('AWS_CLOUD_ELASTIC_IP_REPEAT_COUNT', 3);

/**
 * Tests AWS Cloud Elastic IP.
 *
 * @group AWS Cloud
 */
class ElasticIpTest extends AwsCloudTestCase {

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud elastic ip',
      'add aws cloud elastic ip',
      'view aws cloud elastic ip',
      'edit aws cloud elastic ip',
      'delete aws cloud elastic ip',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'public_ip' => Utils::getRandomPublicIp(),
      'allocation_id' => 'eipalloc-' . $this->getRandomAwsId(),
      'domain' => 'vpc',
    ];
  }

  /**
   * Tests CRUD for Elastic IP information.
   */
  public function testElasticIp() {
    $cloud_context = $this->cloudContext;

    // List Elastic IP for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip");
    $this->assertResponse(200, t('List | HTTP 200: Elastic IP'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Make sure w/o Warnings'));

    $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip/add");

    // Add a new Elastic IP.
    $add = $this->createElasticIpTestData();
    for ($i = 0; $i < AWS_CLOUD_ELASTIC_IP_REPEAT_COUNT; $i++) {
      $this->reloadMockData();
      $domain = $this->getRandomDomain();
      $this->updateDomainInMockData($domain);

      $num = $i + 1;

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/elastic_ip/add",
                            $add[$i],
                            t('Save'));

      $this->assertResponse(200, t('Add | HTTP 200: A New AWS Cloud Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | Make sure w/o Warnings'));
      $this->assertText($add[$i]['name'], t('Elastic IP: @name ', ['@name' => $add[$i]['name']]));
      $this->assertText(t('The AWS Cloud Elastic IP "@name', [
        '@name' => $add[$i]['name'],
      ]),
                        t('Confirm Message') . ': '
                      . t('The AWS Cloud Elastic IP "@name" has been created.', [
                        '@name' => $add[$i]['name'],
                      ]));

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip/$num");
      $this->assertResponse(200, t('Add | View | HTTP 200: Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | View | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | View | Make sure w/o Warnings'));

      // Make sure domain is updated.
      $this->assertText($domain, t('Add | View | Make sure domain: @domain', ['@domain' => $domain]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip");
      $this->assertResponse(200, t('Add | List | HTTP 200: Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$j]['name'],
                        t('Add | List | Make sure w/ Listing: @name', [
                          '@name' => $add[$i]['name'],
                        ]));
      }
    }

    // Edit an Elastic IP information.
    $edit = $this->createElasticIpTestData();
    for ($i = 0; $i < AWS_CLOUD_ELASTIC_IP_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['domain']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/elastic_ip/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->assertResponse(200, t('Edit | HTTP 200: A New AWS Cloud Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Elastic IP "@name" has been saved.', [
        '@name' => $edit[$i]['name'],
      ]),
                        t('Confirm Message') . ': '
                      . t('Edit | The AWS Cloud Elastic IP "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$j]['name'],
                        t('Make sure w/ Listing: @name', [
                          '@name' => $edit[$j]['name'],
                        ]));
      }
    }

    // Delete Elastic IP
    // 3 times.
    $this->updateInstanceInMockData();
    for ($i = 0; $i < AWS_CLOUD_ELASTIC_IP_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip/$num/delete");
      $this->assertResponse(200, t('Delete | HTTP 200: Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/elastic_ip/$num/delete",
                            [],
                            t('Delete'));

      $this->assertResponse(200, t('Delete | HTTP 200: The Cloud Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Elastic IP "@name" has been deleted.', [
        '@name' => $edit[$i]['name'],
      ]),
        t('Confirm Message') . ': '
          . t('Delete | The AWS Cloud Elastic IP "@name" has been deleted.', [
            '@name' => $edit[$i]['name'],
          ])
      );

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/elastic_ip");
      $this->assertResponse(200, t('Delete | List | HTTP 200: Elastic IP #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | List | Make sure w/o Warnings'));

      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertNoText($edit[$j]['name'],
          t('Delete | List | Make sure w/ Listing: @name', [
            '@name' => $edit[$j]['name'],
          ]));
      }
    }
  }

  /**
   * Create elastic ip test data.
   *
   * @return string[][]
   *   Elastic IP array.
   */
  private function createElasticIpTestData() {
    $data = [];
    // 3 times.
    for ($i = 0; $i < AWS_CLOUD_ELASTIC_IP_REPEAT_COUNT; $i++) {
      // Input Fields.
      $num = $i + 1;
      $data[$i] = [
        'name'        => "Elastic IP #$num - " . date('Y/m/d - ') . $this->random->name(15, TRUE),
        'domain'      => 'standard',
      ];
    }
    return $data;
  }

  /**
   * Get Random domain.
   *
   * @return string
   *   a random domain.
   */
  private function getRandomDomain() {
    $domains = ['standard', 'vpc'];
    return $domains[array_rand($domains)];
  }

  /**
   * Update domain in mock data.
   *
   * @param string $domain
   *   a domain.
   */
  private function updateDomainInMockData($domain) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['AllocateAddress']['Domain'] = $domain;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Add an instance into the mock data.  This is used
   * during elastic ip delete testing.
   */
  private function updateInstanceInMockData() {
    $public_ip = Utils::getRandomPublicIp();
    $private_ip = Utils::getRandomPrivateIp();
    $regions = ['us-west-1', 'us-west-2'];
    $region = $regions[array_rand($regions)];

    $vars = [
      // 12 digits.
      'owner_id' => rand(100000000000, 999999999999),
      'reservation_id' => 'r-' . $this->getRandomAwsId(),
      'group_name' => $this->random->name(8, TRUE),
      'host_id' => $this->random->name(8, TRUE),
      'affinity' => $this->random->name(8, TRUE),
      'launch_time' => date('c'),
      'security_group_id' => 'sg-' . $this->getRandomAwsId(),
      'security_group_name' => $this->random->name(10, TRUE),
      'public_dns_name' => Utils::getPublicDns($region, $public_ip),
      'public_ip_address' => $public_ip,
      'private_dns_name' => Utils::getPrivateDns($region, $private_ip),
      'private_ip_address' => $private_ip,
      'vpc_id' => 'vpc-' . $this->getRandomAwsId(),
      'subnet_id' => 'subnet-' . $this->getRandomAwsId(),
      'image_id' => 'ami-' . $this->getRandomAwsId(),
      'reason' => $this->random->string(16, TRUE),
      'instance_id' => 'i-' . $this->getRandomAwsId(),
      'state' => 'running',
    ];
    $instance_mock_data_content = $this->getMockDataFileContent('Drupal\Tests\aws_cloud\Functional\Ec2\InstanceTest', $vars, '_instance');

    $mock_data = $this->getMockDataFromConfig();
    $instance_mock_data = Yaml::decode($instance_mock_data_content);
    $mock_data['DescribeInstances']['Reservations'][0]['OwnerId'] = $this->random->name(8, TRUE);
    $mock_data['DescribeInstances']['Reservations'][0]['ReservationId'] = $this->random->name(8, TRUE);
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][] = $instance_mock_data;
    $this->updateMockDataToConfig($mock_data);
  }

}
